<?php
namespace App\Factory;

use App\Exception\ParamsException as ParamsException;

class OrderFactory {
    
    private $connection;
    private $util;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util) {
        $this->connection = $connection;
        $this->util       = $util;
    }
    
    public function getFilial($cdFilial, $cdExpFilial) {
        if((isset($cdExpFilial)) && (!empty($cdExpFilial))) {
            $subsidiaryCode = $cdExpFilial;
        } else {
            $subsidiaryCode = $cdFilial; 
        }
        
        $sql = "
                SELECT F.CDFILIAL, F.NMFILIAL 
                 FROM FILIAL F
                 LEFT JOIN PARAMFILIAL PF
                    ON F.CDFILIAL = PF.CDFILIAL
                 WHERE (F.CDFILIAL = :CDFILIAL OR PF.CDEXFILIAL = :CDFILIAL)
        ";
        
        $params = [
                ':CDFILIAL' => $cdFilial,
                ':CDEXFILIAL' => $cdExpFilial
            ];
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getBranchDetails($cdFilial, $cdExpFilial) {
        $sql = "
                SELECT F.CDFILIAL, F.NMFILIAL 
                 FROM FILIAL F
                 LEFT JOIN PARAMFILIAL PF
                    ON F.CDFILIAL = PF.CDFILIAL
                 WHERE (F.CDFILIAL = :CDFILIAL OR PF.CDEXFILIAL = :CDEXFILIAL)
        ";
        
        $params = [
                ':CDFILIAL' => $cdFilial,
                ':CDEXFILIAL' => $cdExpFilial
            ];
        $branch =  $this->connection->fetchAssoc($sql, $params);
        
        if(empty($branch)) {
             throw new Exception\ParamsException("Filial não encontrada na base. É obrigatório o envio do código de filial Teknisa no campo subsidiaryCode ou do código no sistema parceiro no campo subsidiaryExtCode.");
        }
        
        $sql = "
            SELECT F.CDFILIAL, F.NMFILIAL, F.NMRAZSOCFILI, F.NRINSJURFILI, IE.NRINSJURIEST, E.CDEMPRESA, PE.CDSITUCRT, F.CDINSCESTA,
                   F.NRORG, PV.CDCLIENTE, PV.CDVENDEDOR, EF.CDPAIS, EF.SGESTADO, EF.CDMUNICIPIO, EF.DSENDEFILI, 
                   EF.NMBAIRFILI, EF.NRCEPFILI, EF.NRENDEFILISEP, V.CDCAIXA, V.CDOPERADOR, C.CDLOJA, C.IDTPEMISSAOFOS
            FROM FILIAL F
                JOIN PARAVEND PV
                    ON PV.CDFILIAL = F.CDFILIAL
                JOIN ENDEFILI EF
                    ON EF.CDFILIAL = F.CDFILIAL
                JOIN EMPRESA E
                    ON E.CDEMPRESA = F.CDEMPRESA
                JOIN PARAMEMPRESA PE
                    ON PE.CDEMPRESA = E.CDEMPRESA
                JOIN INSCRESTAD IE
                    ON IE.CDEMPRESA = E.CDEMPRESA
                    AND IE.CDINSCESTA = F.CDINSCESTA
                LEFT JOIN VENDEDOR V
                    ON V.CDVENDEDOR = PV.CDVENDEDOR
                LEFT JOIN CAIXA C
                    ON C.CDCAIXA = V.CDCAIXA
                    AND C.CDFILIAL = F.CDFILIAL
            WHERE F.CDFILIAL = :CDFILIAL
            AND EF.IDTPENDEFILI = 'P'
        ";
        
        $params = [
                ':CDFILIAL' => $branch['CDFILIAL']
            ];
            
        return $this->connection->fetchAssoc($sql, $params);
        
    }
    
    public function getSales($subsidiaryCode, $dtInicial, $dtFinal) {
        $sql = "
                SELECT V.CDFILIAL,V.NRINSJURIEST, V.CDCAIXA, V.NRSEQVENDA, TO_CHAR(V.DTVENDA, 'dd-mm-yyyy hh24:mi:ss') AS DTVENDA,
                       TO_CHAR(V.DTENTRVENDA, 'dd-mm-yyyy') AS DTENTRVENDA, TO_CHAR(V.DTEMISSAONFCE, 'dd-mm-yyyy hh24:mi:ss') AS DTEMISSAONFCE,
                       TO_CHAR(V.DTABERTUR, 'dd-mm-yyyy') AS DTABERTUR, V.IDSITUVENDA, V.IDSTATUSNFCE, V.NRACESSONFCE, 
                       V.NRACESSOCANC, V.NRNOTAFISCALCE, V.CDSERIENFCE, V.IDCANCMANUAL, V.SGESTADO, V.NRLANCTONFCE, V.DSOBSSTATUSNFCE,
                       V.DSOBSCANCUP, V.NRPROTOCOLONFCE, NRPROTOCOLOCANC, V.VRTOTVENDA, V.VRTXSEVENDA, V.CDOPERADOR, V.NRINSCRCONS,
                       V.CDCLIENTE, V.CDCONSUMIDOR, O.NMOPERADOR, C.NMCAIXA, PC.NRINSCRICAOPARC
                FROM VENDA V
                    JOIN OPERADOR O
                        ON O.CDOPERADOR = V.CDOPERADOR
                    JOIN CAIXA C
                        ON C.CDCAIXA = V.CDCAIXA
                        AND C.CDFILIAL = V.CDFILIAL
                        AND C.CDLOJA = V.CDLOJA
                    LEFT JOIN PARCNEGOCIO PC
                        ON PC.NRPARCNEGOCIO = O.NRPARCNEGOCIO
                    WHERE V.CDFILIAL = :CDFILIAL
                        AND V.IDTPAMBNFCE = '2'
                        AND V.DTVENDA BETWEEN TO_DATE(:DATA_INI,'dd/mm/yyyy hh24:mi:ss') AND TO_DATE(:DATA_FIM,'dd/mm/yyyy hh24:mi:ss')
                        AND V.NRNOTAFISCALCE IS NOT NULL
                ORDER BY DTVENDA, CDCAIXA    
                
                ";
        
        
        $params = [
            ':CDFILIAL' => $subsidiaryCode,
            ':DATA_INI' => $dtInicial,
            ':DATA_FIM' => $dtFinal
            ];
        
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getVrTotalVenda($subsidiaryCode, $cdCaixa , $nrSeqVenda) {
        
        $sql = "
                SELECT SUM(SUM((QTPRODVEND*VRUNITVEND)-NVL(VRDESITVEND,0)+NVL(VRACRITVEND,0))) AS VRTOTAL
                FROM ITEMVENDA 
                    WHERE CDFILIAL = :CDFILIAL
                    AND CDCAIXA    = :CDCAIXA
                    AND NRSEQVENDA = :NRSEQVENDA
                    
                    GROUP BY NRSEQVENDA 
                ";
                
        $params = [
                ':CDFILIAL'   => $subsidiaryCode,
                ':CDCAIXA'    => $cdCaixa,
                ':NRSEQVENDA' => $nrSeqVenda
                ];
                
        return $this->connection->fetchAssoc($sql, $params);
    }
    
     public function getTotalAcreDesc($subsidiaryCode, $cdCaixa , $nrSeqVenda) {
        
        $sql = "
                SELECT SUM(VRDESITVEND) AS VRDESITVEND,
                       SUM(VRACRITVEND) AS VRACRITVEND
                FROM ITEMVENDA
                WHERE CDFILIAL = :CDFILIAL
                    AND CDCAIXA    = :CDCAIXA
                    AND NRSEQVENDA = :NRSEQVENDA
                ";
                
        $params = [
                ':CDFILIAL'   => $subsidiaryCode,
                ':CDCAIXA'    => $cdCaixa,
                ':NRSEQVENDA' => $nrSeqVenda
                ];
                
        $result = $this->connection->fetchAssoc($sql, $params);
        
        $total = [
                'acrescimo' => isset($result['VRACRITVEND']) ? $result['VRACRITVEND'] : 0 ,
                'desconto'  => isset($result['VRDESITVEND']) ? $result['VRDESITVEND'] : 0 
            ];

        return $total;
    }
    
    public function getTotalImpostos($subsidiaryCode, $cdCaixa , $nrSeqVenda) {
        
        $sql = "
                SELECT SUM(VRIMPPIS) AS VRIMPPIS,
                       SUM(VRIMPCOFINS) AS VRIMPCOFINS, 
                       SUM(VRIMPOPRODIT) AS VRIMPOPRODIT
                FROM ITVENDAIMPOS
                WHERE CDFILIAL = :CDFILIAL
                    AND CDCAIXA    = :CDCAIXA
                    AND NRSEQVENDA = :NRSEQVENDA
                ";
                
        $params = [
                    ':CDFILIAL'   => $subsidiaryCode,
                    ':CDCAIXA'    => $cdCaixa,
                    ':NRSEQVENDA' => $nrSeqVenda
                ];
        
        $result = $this->connection->fetchAssoc($sql, $params);
        
        $total = [
                    'vrPIS'    => isset($result['VRIMPPIS']) ? $result['VRIMPPIS'] : 0,
                    'vrCOFINS' => isset($result['VRIMPCOFINS']) ? $result['VRIMPCOFINS'] : 0,
                    'vrICMS'   => isset($result['VRIMPOPRODIT']) ? $result['VRIMPOPRODIT'] : 0
                ];
        
        return $total;
    }
    
    public function getClosingType($subsidiaryCode) {
        $sql = "
                SELECT IDTPAPUFIL FROM PARAVEND
                WHERE CDFILIAL = :CDFILIAL
        ";
        $params = [
                    ':CDFILIAL' => $subsidiaryCode
                ];
            
        return $this->connection->fetchAssoc($sql, $params);
        
    }
    
    public function getTipoSangria($subsidiaryCode) {
        $sql = "
                SELECT CDTPSANGRIA 
                    FROM TIPOSANGRIA
                WHERE CDFILIAL = :CDFILIAL
                    AND IDSANGRIA = 'F'
        ";
        $params = [
                ':CDFILIAL' => $subsidiaryCode
                ];
            
        return $this->connection->fetchAssoc($sql, $params);
        
    }
    
    public function closingPosBySangria($subsidiaryCode, $dtInicial, $dtFinal, $cdTpSangria) {
        $sql = "
                SELECT TO_DATE(DTABERCAIX,'DD-MM-YYYY hh24:mi:ss') AS DTABERCAIX, CDTIPORECE, SUM(VRMOVIVEND) AS VRMOVIVEND
                    FROM MOVCAIXA
                 WHERE CDFILIAL = :CDFILIAL
                        AND DTABERCAIX BETWEEN TO_DATE(:DATA_INI, 'DD/MM/YYYY hh24:mi:ss') AND TO_DATE(:DATA_FIM, 'DD/MM/YYYY hh24:mi:ss')
                        AND IDTIPOMOVIVE = 'G'
                        AND CDTPSANGRIA = :CDTPSANGRIA
                GROUP BY DTABERCAIX, CDTIPORECE
        ";
        $params = [
                    ':CDFILIAL'    => $subsidiaryCode,
                    ':DATA_INI'    => $dtInicial,
                    ':DATA_FIM'    => $dtFinal,
                    ':CDTPSANGRIA' => $cdTpSangria['CDTPSANGRIA']
                ];

        return $this->connection->fetchAll($sql, $params);
    }
    
    public function closingPosByVenda($subsidiaryCode, $dtInicial, $dtFinal) {
        $sql = "
                SELECT TO_DATE(DTABERCAIX,'DD-MM-YYYY hh24:mi:ss') AS DTABERCAIX,
                        CDTIPORECE, SUM(CASE WHEN IDTIPOMOVIVE = 'E' 
                                     THEN VRMOVIVEND 
                                     ELSE - VRMOVIVEND END ) AS VRMOVIVEND
                   FROM MOVCAIXA
                     WHERE CDFILIAL = :CDFILIAL
                        AND DTABERCAIX BETWEEN TO_DATE(:DATA_INI, 'DD/MM/YYYY hh24:mi:ss') AND TO_DATE(:DATA_FIM, 'DD/MM/YYYY hh24:mi:ss')
                        AND IDTIPOMOVIVE IN ('E','S')
                    GROUP BY DTABERCAIX, CDTIPORECE
                ORDER BY DTABERCAIX, CDTIPORECE
        ";

        $params = [
                    ':CDFILIAL'    => $subsidiaryCode,
                    ':DATA_INI'    => $dtInicial,
                    ':DATA_FIM'    => $dtFinal
            ];
        
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getPaymentInfo($cdTipoRece) {
        $sql = "
                SELECT TR.CDTIPORECE, TR.NMTIPORECE, TR.IDTIPORECE, RI.CDTPRECEINTE
                    FROM TIPORECE TR
                  LEFT JOIN (SELECT TP.CDTPRECEINTE, TP.CDTIPORECE    
                        FROM TIPORECEINTE TP
                      JOIN INTEGRACAO INTE
                         ON TP.CDINTEGRACAO = INTE.CDINTEGRACAO
                      WHERE INTE.IDINTEGRACAO = 'DTS'
                        AND TP.CDTIPORECE = :CDTIPORECE) RI 
                    ON TR.CDTIPORECE = RI.CDTIPORECE
                WHERE TR.CDTIPORECE = :CDTIPORECE

        ";
        $params = [
                    ':CDTIPORECE'  => $cdTipoRece
                ];

        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getPlanRequest($subsidiaryCode, $dtInicial, $dtFinal) {
        $sql = "
               SELECT DISTINCT NRLANCESTQ FROM RETIRADA
                    WHERE CDFILIAL = :CDFILIAL
                        AND DTRETIRADA BETWEEN TO_DATE(:DATA_INI,'DD/MM/YYYY') AND TO_DATE(:DATA_FIM,'DD/MM/YYYY')

            ";
        $params = [
                ':CDFILIAL' => $subsidiaryCode,
                ':DATA_INI' => date("d-m-Y", strtotime($dtInicial)),
                ':DATA_FIM' => date("d-m-Y", strtotime($dtFinal))
                ];
        
        $sql = $this->connection->fetchAll($sql, $params);
        
        return $sql;
    }
    
    public function getSaleRequest($subsidiaryCode, $dtInicial, $dtFinal) {
        $sql = "
               SELECT DISTINCT(IV.NRLANCESTQ) FROM VENDA V
                    JOIN ITEMVENDA IV
                    ON V.CDFILIAL = IV.CDFILIAL
                        AND V.CDCAIXA = IV.CDCAIXA
                        AND TO_DATE(V.DTENTRVENDA, 'DD/MM/YYYY') = TO_DATE(IV.DTHRPEDIDO, 'DD/MM/YYYY')
                WHERE V.CDFILIAL = :CDFILIAL
                    AND V.DTENTRVENDA BETWEEN TO_DATE(:DATA_INI, 'DD/MM/YYYY') AND TO_DATE(:DATA_FIM, 'DD/MM/YYYY')
                    AND IV.NRLANCESTQ IS NOT NULL
            ";
        $params = [
                ':CDFILIAL' => $subsidiaryCode,
                ':DATA_INI' => date("d-m-Y", strtotime($dtInicial)),
                ':DATA_FIM' => date("d-m-Y", strtotime($dtFinal))
                ];
        
        $sql = $this->connection->fetchAll($sql, $params);

        return $sql;
    }
    
    public function getMovEstoq($subsidiaryCode, $nrLancEstq) {
        $sql = "
               SELECT IT.CDPRODUTO, P.NMPRODUTO, IT.NRLANCESTQ, IT.QTLANCTOEST, TO_DATE(IT.DTLANCMOVI, 'DD-MM-YYYY') AS DTLANCMOVI,
                       P.CDARVPROD, P.CDPRODINTE, P.SGUNIDADE, IV.CDPRODUTO AS PRODPAI, IV.NRSEQVENDA, V.NRNOTAFISCALCE,
                       NVL(RFL.QTSUBPRODLOC, RF.QTSUBPRODROT) AS QTDCOMP, IV.QTPRODVEND
                                FROM ITLANCTOEST IT
                    JOIN PRODUTO P
                        ON IT.CDPRODUTO = P.CDPRODUTO
                    LEFT JOIN ITEMVENDA IV
                        ON IV.CDFILIALEST  = IT.CDFILIMOVI
                         AND IV.NRLANCESTQ = IT.NRLANCESTQ
                         AND IV.CDPRODUTO  IN (SELECT NVL(NVL(RF.CDPRODUTO, RFL.CDPRODUTO), IT.CDPRODUTO) AS PRODPAI FROM PRODUTO P
                                                    LEFT JOIN ROTFABRICA RF
                                                        ON RF.CDSUBPRODUTO = P.CDPRODUTO
                                                    LEFT JOIN ROTFABLOCAL RFL
                                                        ON RFL.CDSUBPRODLOC = P.CDPRODUTO
                                                 WHERE P.CDPRODUTO = IT.CDPRODUTO)
                         AND IV.CDFILIAL   = IT.CDFILIMOVI
                    LEFT JOIN VENDA V
                        ON V.NRSEQVENDA = IV.NRSEQVENDA
                         AND V.CDFILIAL = IT.CDFILIMOVI
                    LEFT JOIN ROTFABRICA RF
                        ON IV.CDPRODUTO = RF.CDPRODUTO
                        AND IT.CDPRODUTO = RF.CDSUBPRODUTO
                    LEFT JOIN ROTFABLOCAL RFL
                        ON RFL.CDPRODUTO = IV.CDPRODUTO
                        AND RFL.CDSUBPRODLOC = IT.CDPRODUTO
                        AND RFL.CDFILIAL = IV.CDFILIAL
                    WHERE IT.CDFILIMOVI = :CDFILIAL
                        AND IT.NRLANCESTQ = :NRLANCESTQ
            ";
        $params = [
                ':CDFILIAL' => $subsidiaryCode,
                ':NRLANCESTQ' => $nrLancEstq
                ];
        
        $sql = $this->connection->fetchAll($sql, $params);

        return $sql;
    }

    public function getDateMovEst($subsidiaryCode, $nrLancEstq) {
        $sql = "
               SELECT TO_CHAR(DTLANCESTQ, 'DD-MM-YYYY') AS DTLANCESTQ FROM LANCTOESTOQ
                    WHERE CDFILIMOVI = :CDFILIAL
                    AND NRLANCESTQ = :NRLANCESTQ
            ";
        $params = [
                ':CDFILIAL' => $subsidiaryCode,
                ':NRLANCESTQ' => $nrLancEstq
                ];
        
        $sql = $this->connection->fetchAssoc($sql, $params);

        return $sql['DTLANCESTQ'];
    }
    
    public function getUnutilizedNotes($subsidiaryCode, $dtInicial, $dtFinal) {
        $sql = "
                SELECT CDFILIAL, NRNOTAFISCALCE, DTENTRVENDA, CDSERIENFCE, IDTPAMBNFCE, IDSTATUSNFCE,
                       DTHRINUTNFCE, IDTPEMISINUNFCE, DSRAZAOINUTNFCE, DSOBSINUTNFCE, NRINSJURFILI
                  FROM INUTILIZANFCE
                    WHERE CDFILIAL = :CDFILIAL
                        AND IDTPAMBNFCE = '1'
                        AND DTENTRVENDA BETWEEN TO_DATE(:DATA_INI,'dd/mm/yyyy') AND TO_DATE(:DATA_FIM,'dd/mm/yyyy')
                ORDER BY DTENTRVENDA 

        ";

        $params = [
            ':CDFILIAL' => $subsidiaryCode,
            ':DATA_INI' => date("d-m-Y", strtotime($dtInicial)),
            ':DATA_FIM' => date("d-m-Y", strtotime($dtFinal))
            ];
            
        
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getconsumerDetails($cdFilial, $cdCliente, $cdConsumidor, $cpf) {
        $sql = "
              SELECT C.CDCLIENTE, C.CDCONSUMIDOR, C.NMCONSUMIDOR, C.DSENDECONS,
                     C.NMBAIRCONS, C.NRCEPCONS, C.NRENDECONS, C.DSEMAILCONS, C.CDPAIS,
                     P.NMPAIS, C.SGESTADO, C.CDMUNICIPIO, M.NMMUNICIPIO
                FROM CONSUMIDOR C
                  JOIN PAIS P
                    ON P.CDPAIS = C.CDPAIS
                  JOIN MUNICIPIO M
                    ON M.CDMUNICIPIO = C.CDMUNICIPIO
                WHERE NRCPFRESPCON = :NRCPFRESPCON
        ";
        
        $params = [
                ':NRCPFRESPCON' => str_replace("-", "", str_replace(".", "", $cpf))
            ];
        
        $consumer = $this->connection->fetchAll($sql, $params);
        
        if((empty($consumer)) || (count($consumer) > 1)) {
            $sql = "
                    SELECT CDCONSUMIDOR, CDCONSUMIDPADRAO 
                        FROM PARAVEND
                         WHERE CDFILIAL    = :CDFILIAL
                          AND CDCLIENTE    = :CDCLIENTE
                          AND CDCONSUMIDOR = :CDCONSUMIDOR
                ";
            $params = [
                ':CDFILIAL'  => $cdFilial,
                ':CDCLIENTE' => $cdCliente,
                ':CDCONSUMIDOR' => $cdConsumidor
            ];
            
            $result = $this->connection->fetchAssoc($sql, $params);
            if((isset($result['CDCONSUMIDOR'])) || (isset($result['CDCONSUMIDPADRAO']))){
                return null;
            } else {
                $sql = "
                    SELECT C.CDCLIENTE, C.CDCONSUMIDOR, C.NMCONSUMIDOR, C.DSENDECONS,
                           C.NMBAIRCONS, C.NRCEPCONS, C.NRENDECONS, C.DSEMAILCONS, C.CDPAIS,
                           P.NMPAIS, C.SGESTADO, C.CDMUNICIPIO, M.NMMUNICIPIO
                    FROM CONSUMIDOR C
                      JOIN PAIS P
                        ON P.CDPAIS = C.CDPAIS
                      JOIN MUNICIPIO M
                        ON M.CDMUNICIPIO = C.CDMUNICIPIO
                     WHERE CDCLIENTE    = :CDCLIENTE
                      AND CDCONSUMIDOR = :CDCONSUMIDOR
                ";
                $params = [
                    ':CDCLIENTE' => $cdCliente,
                    ':CDCONSUMIDOR' => $cdConsumidor
                ];
                
                return $this->connection->fetchAssoc($sql, $params);
            }
        }
    }
    
    public function getDayCloses ($cdFilial, $dtMovTurCaix) {
        
        $sql = "
            SELECT T.CDFILIAL, T.CDCAIXA, CX.NMCAIXA, TO_CHAR(T.DTABERCAIX,'DD-MM-YYYY hh24:mi:ss') AS DTABERCAIX,
                    T.CDOPERADOR, O.NMOPERADOR, TO_CHAR(T.DTFECHCAIX,'DD-MM-YYYY hh24:mi:ss') AS DTFECHCAIX, 
                    T.CDOPERFECH, TO_CHAR(T.DTMOVTURCAIX,'DD-MM-YYYY') AS DTMOVTURCAIX, PC.NRINSCRICAOPARC
                FROM TURCAIXA T
                    JOIN CAIXA CX
                        ON CX.CDFILIAL = T.CDFILIAL
                        AND CX.CDCAIXA = T.CDCAIXA
                    JOIN OPERADOR O
                        ON O.CDOPERADOR = T.CDOPERADOR
                    LEFT JOIN PARCNEGOCIO PC
                        ON PC.NRPARCNEGOCIO = O.NRPARCNEGOCIO
                    WHERE T.CDFILIAL = :CDFILIAL
                     AND T.DTMOVTURCAIX = :DTMOVTURCAIX
                     AND T.DTFECHCAIX IS NOT NULL
        ";
        
        $params = [
            ':CDFILIAL'     => $cdFilial,
            ':DTMOVTURCAIX' => $dtMovTurCaix
        ];
        
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getClosingSummary ($cdFilial, $cdCaixa, $dtAberCaix) {
        
        $sql = "
            SELECT MV.CDFILIAL, MV.CDCAIXA, SUM(MV.VRMOVIVEND) AS VRMOVIVEND, MV.CDTIPORECE
                FROM MOVCAIXA MV
                    LEFT JOIN TIPOSANGRIA TS
                        ON TS.CDTPSANGRIA = MV.CDTPSANGRIA
                 WHERE MV.CDFILIAL   = :CDFILIAL
                    AND MV.CDCAIXA      = :CDCAIXA
                    AND MV.DTABERCAIX   = TO_DATE(:DTABERCAIX, 'dd/mm/yyyy hh24:mi:ss')
                    AND MV.IDTIPOMOVIVE = 'G'
                    AND (TS.IDSANGRIA <> 'D' OR TS.IDSANGRIA IS NULL)
            GROUP BY MV.CDFILIAL, MV.CDCAIXA, MV.CDTIPORECE

        ";
        
        $params = [
            ':CDFILIAL'   => $cdFilial,
            ':CDCAIXA'    => $cdCaixa,
            ':DTABERCAIX' => $dtAberCaix
        ];

        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getClosingSummaryBySale ($cdFilial, $cdCaixa, $dtAberCaix) {
        
        $sql = "
            SELECT
              A.CDTIPORECE, A.NMTIPORECE, A.IDTIPORECE, NVL(A.VRMOVIVEND,0) -  NVL(B.VRMOVIVEND,0) AS VRMOVIVEND
            FROM
            (SELECT
              T.CDTIPORECE, T.NMTIPORECE, T.IDTIPORECE, SUM(M.VRMOVIVEND) AS VRMOVIVEND
            FROM
              MOVCAIXA M, TIPORECE T
            WHERE M.CDFILIAL      = :CDFILIAL
              AND M.CDCAIXA       = :CDCAIXA
              AND (M.IDTIPOMOVIVE  IN ('E','T','U'))
              AND M.DTABERCAIX    = TO_DATE(:DTABERCAIX, 'dd/mm/yyyy hh24:mi:ss')
              AND M.CDTIPORECE    = T.CDTIPORECE
              AND T.IDTIPORECE    <> '7'
              AND T.IDTIPORECE    <> '5'
            GROUP BY
              T.CDTIPORECE, T.NMTIPORECE, T.IDTIPORECE) A LEFT JOIN
            (SELECT
              T.CDTIPORECE, T.NMTIPORECE, T.IDTIPORECE, SUM(M.VRMOVIVEND) AS VRMOVIVEND
            FROM
              MOVCAIXA M, TIPORECE T
            WHERE M.CDFILIAL      = :CDFILIAL
              AND M.CDCAIXA       = :CDCAIXA
              AND M.IDTIPOMOVIVE  = 'S'
              AND M.DTABERCAIX    = TO_DATE(:DTABERCAIX, 'dd/mm/yyyy hh24:mi:ss')
              AND M.CDTIPORECE    = T.CDTIPORECE
              AND T.IDTIPORECE    <> '7'
              AND T.IDTIPORECE    <> '5'
            GROUP BY
              T.CDTIPORECE, T.NMTIPORECE, T.IDTIPORECE) B
              ON (A.CDTIPORECE = B.CDTIPORECE)
            UNION ALL
            SELECT
              T.CDTIPORECE, T.NMTIPORECE, T.IDTIPORECE, SUM(M.VRMOVIVEND) AS VRMOVIVEND
            FROM
              MOVCAIXA M, TIPORECE T
            WHERE M.CDFILIAL      = :CDFILIAL
              AND M.CDCAIXA       = :CDCAIXA
              AND M.IDTIPOMOVIVE  = 'E'
              AND M.DTABERCAIX    = TO_DATE(:DTABERCAIX, 'dd/mm/yyyy hh24:mi:ss')
              AND M.CDTIPORECE    = T.CDTIPORECE
              AND ((T.IDTIPORECE   = '7') OR (T.IDTIPORECE = '5'))
            GROUP BY
              T.CDTIPORECE, T.NMTIPORECE, T.IDTIPORECE
        ";
        
        $params = [
            ':CDFILIAL'   => $cdFilial,
            ':CDCAIXA'    => $cdCaixa,
            ':DTABERCAIX' => $dtAberCaix
        ];

        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getAbertur($cdFilial, $dtInicial, $dtFinal) {
        $sql = "
            SELECT TC.CDFILIAL, TC.CDCAIXA, TO_CHAR(TC.DTABERCAIX,'DD-MM-YYYY hh24:mi:ss') AS DTABERCAIX,
                   TO_CHAR(TC.DTFECHCAIX,'DD-MM-YYYY hh24:mi:ss') AS DTFECHCAIX, TC.CDOPERADOR, O.NMOPERADOR,
                   PC.NRINSCRICAOPARC
              FROM TURCAIXA TC
                JOIN OPERADOR O
                    ON O.CDOPERADOR = TC.CDOPERADOR
                LEFT JOIN PARCNEGOCIO PC
                    ON PC.NRPARCNEGOCIO = O.NRPARCNEGOCIO
                WHERE TC.CDFILIAL = :CDFILIAL
                  AND TC.DTFECHCAIX BETWEEN TO_DATE(:DATA_INI, 'dd/mm/yyyy hh24:mi:ss') AND TO_DATE(:DATA_FIM, 'dd/mm/yyyy hh24:mi:ss')
        ";
        
        $params = [
            ':CDFILIAL' => $cdFilial,
            ':DATA_INI' => $dtInicial,
            ':DATA_FIM' => $dtFinal
        ];

        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getCashWithdrawal ($cdFilial, $cdCaixa, $dtAberCaix) {
        
        $sql = "
            SELECT MV.CDFILIAL, MV.CDCAIXA, MV.VRMOVIVEND, MV.CDTIPORECE,
                   MV.CDTPSANGRIA, TS.NMTPSANGRIA, MV.DSOBSSANGRIACX
                FROM MOVCAIXA MV
                    LEFT JOIN TIPOSANGRIA TS
                        ON TS.CDTPSANGRIA = MV.CDTPSANGRIA
                 WHERE MV.CDFILIAL   = :CDFILIAL
                    AND MV.CDCAIXA      = :CDCAIXA
                    AND MV.DTABERCAIX   = TO_DATE(:DTABERCAIX, 'dd/mm/yyyy hh24:mi:ss')
                    AND MV.IDTIPOMOVIVE = 'G'
                    AND TS.IDSANGRIA = 'D'
        ";
        
        $params = [
            ':CDFILIAL'   => $cdFilial,
            ':CDCAIXA'    => $cdCaixa,
            ':DTABERCAIX' => $dtAberCaix
        ];
        
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getCashSupply ($cdFilial, $cdCaixa, $dtAberCaix) {
        
        $sql = "
            SELECT SUM(VRMOVIVEND) AS VRMOVIVEND
                FROM MOVCAIXA
                 WHERE CDFILIAL = :CDFILIAL
                  AND DTABERCAIX = TO_DATE(:DTABERCAIX, 'DD/MM/YYYY hh24:mi:ss')
                  AND IDTIPOMOVIVE IN ('A', 'U')
            GROUP BY DTABERCAIX, CDTIPORECE
        ";
        
        $params = [
            ':CDFILIAL'   => $cdFilial,
            ':CDCAIXA'    => $cdCaixa,
            ':DTABERCAIX' => $dtAberCaix
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getTotalSaleSubsidized($cdFilial, $cdCaixa, $nrSeqVenda, $cdCliente) {
        
        $sql = "
            SELECT SUM(VRMOVCLI) AS VRMOVCLI
                FROM MOVICLIE
                 WHERE CDFILIAL = :CDFILIAL
                  AND CDCAIXA = :CDCAIXA
                  AND CDCLIENTE = :CDCLIENTE
                  AND NRSEQVENDA = :NRSEQVENDA
                  AND CDCONSUMIDOR IS NULL
            GROUP BY  NRSEQVENDA, CDCLIENTE
        ";
        
        $params = [
            ':CDCLIENTE'  => $cdCliente,
            ':NRSEQVENDA' => $nrSeqVenda,
            ':CDFILIAL'   => $cdFilial,
            ':CDCAIXA'    => $cdCaixa
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getSaleSubsidized($cdFilial, $cdCaixa, $nrSeqVenda, $cdCliente) {
        
        $sql = "
            SELECT NRSEQMOVCLI
                FROM MOVICLIE
                 WHERE CDFILIAL = :CDFILIAL
                  AND CDCAIXA = :CDCAIXA
                  AND CDCLIENTE = :CDCLIENTE
                  AND NRSEQVENDA = :NRSEQVENDA
                  AND CDCONSUMIDOR IS NULL
        ";
        
        $params = [
            ':CDCLIENTE'  => $cdCliente,
            ':NRSEQVENDA' => $nrSeqVenda,
            ':CDFILIAL'   => $cdFilial,
            ':CDCAIXA'    => $cdCaixa
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getPosStatus($cdFilial, $cdCaixa, $dtAberCaix) {
        /*
        1 - Caixa Aberto
        2 - Caixa Fechado
        3 - Caixa Aprovado (Fechamento de Cupom realizado)
        */
        
        $sql = "
            SELECT TO_CHAR(DTABERCAIX,'DD-MM-YYYY hh24:mi:ss') AS DTABERCAIX, TO_CHAR(DTFECHCAIX,'DD-MM-YYYY hh24:mi:ss') AS DTFECHCAIX,
                   CDOPERADOR, CDOPERABER, CDOPERFECH, IDULTFECHAMEN
                FROM TURCAIXA
                WHERE CDFILIAL = :CDFILIAL 
                   AND CDCAIXA = :CDCAIXA
                   AND TRUNC(DTABERCAIX) = :DTABERCAIX
        ";
        
        $params = [
                ':CDFILIAL'   => $cdFilial,
                ':CDCAIXA'    => $cdCaixa,
                ':DTABERCAIX' => $dtAberCaix
            ];
            
        $result =  $this->connection->fetchAssoc($sql, $params);
        
        $reOpen = true;
        $status = null;
        
        if(empty($result)) {
            return ['status'  => $status];
        }
        
        if(empty($result['DTFECHCAIX']) && empty($result['CDOPERFECH'])) {
            $status = 'Aberto';
        } else {
            $status = 'Fechado';
            if($result['IDULTFECHAMEN'] == 'S') {
                $reOpen = false;
            }
        }
        
        
        if($status == 'Fechado' && !$reOpen) {
            $sql = "
                SELECT * FROM VENDA 
                  WHERE CDFILIAL = :CDFILIAL
                    AND CDCAIXA = :CDCAIXA
                    AND TRUNC(DTABERTUR) = :DTABERCAIX
                    AND IDAPROVACAONFCE  = 'A'
                    AND ROWNUM = 1
            ";
            
            $checkVenda = $this->connection->fetchAssoc($sql, $params);
            
            $sql = "
                SELECT * FROM MOVCAIXA 
                  WHERE CDFILIAL = :CDFILIAL
                    AND CDCAIXA = :CDCAIXA
                    AND TRUNC(DTABERCAIX) = :DTABERCAIX
                    AND IDAPROVREDZNFCE = 'A'
                    AND ROWNUM = 1
            ";
            
            $checkMovCaixa = $this->connection->fetchAssoc($sql, $params);
            
            if (!empty($checkVenda) || !empty($checkMovCaixa)) {
                $status = 'Aprovado';
                $reOpen = false;
            }
        }
        
        $return = [
                'status'          => $status,
                'allowsReopening' => $reOpen,
                'openingPos'      => $result['DTABERCAIX']
            ];
            
        return $return;
    }
    
    public function openingPos($cdFilial, $cdCaixa, $dtAberCaix, $dtMovTurCaix, $cdOperador, $nrOrg) {
        $sql = "
            INSERT INTO TURCAIXA
              (CDFILIAL, CDCAIXA, DTABERCAIX, CDOPERADOR,
              CDOPERABER, DTMOVTURCAIX, NRORG, IDULTFECHAMEN)
            VALUES
              (:CDFILIAL, :CDCAIXA, TO_DATE(:DTABERCAIX, 'dd/mm/yyyy hh24:mi:ss'), :CDOPERADOR,
              :CDOPERABER, TO_DATE(:DTMOVTURCAIX, 'dd/mm/yyyy'), :NRORG, :IDULTFECHAMEN)
        ";
        
        $params = [
            ':CDFILIAL'      => $cdFilial,
            ':CDCAIXA'       => $cdCaixa,
            ':DTABERCAIX'    => $dtAberCaix,
            ':CDOPERADOR'    => $cdOperador,
            ':CDOPERABER'    => $cdOperador,
            ':DTMOVTURCAIX'  => $dtMovTurCaix,
            ':NRORG'         => $nrOrg,
            ':IDULTFECHAMEN' => 'N'
        ];
        
        $this->connection->executeQuery($sql, $params);
        
        unset($sql);
        unset($params);
        $nrSequMovi = $this->util->newCode('MOVCAIXA'.$cdFilial.$cdCaixa.$dtAberCaix, 10);
        
        $sql = "
            INSERT INTO MOVCAIXA
             ( CDFILIAL, CDCAIXA, DTABERCAIX, NRSEQUMOVI,
               DTHRINCMOV, IDTIPOMOVIVE, VRMOVIVEND,
               NRSEQVENDA, NRORG, NRORGINCLUSAO, CDOPERINCLUSAO,
               CDTIPORECE, DTMOVIMCAIXA
             )
            VALUES
             ( :CDFILIAL, :CDCAIXA, :DTABERCAIX, :NRSEQUMOVI,
               :DTHRINCMOV, :IDTIPOMOVIVE, :VRMOVIVEND,
               :NRSEQVENDA, :NRORG, :NRORGINCLUSAO, :CDOPERINCLUSAO,
               :CDTIPORECE, :DTMOVIMCAIXA
             )
        ";
        
        $params = [
			'CDFILIAL' 	   	 => $cdFilial,
			'CDCAIXA' 	   	 => $cdCaixa,
			'DTABERCAIX'   	 => $dtAberCaix,
			'NRSEQUMOVI'   	 => $nrSequMovi,
			'DTHRINCMOV'	 => $dtAberCaix,
			'IDTIPOMOVIVE' 	 => 'A',
			'VRMOVIVEND'   	 => 0,
			'QTPARCRECEB'    => '1',
			'NRSEQVENDA'   	 => null,
			'NRORG'			 => $nrOrg,
			'CDTIPORECE'  	 => '001',
			'DTMOVIMCAIXA'	 => $dtAberCaix,
			'CDOPERINCLUSAO' => $cdOperador,
			'NRORGINCLUSAO'  => $nrOrg
		];
		
		$this->connection->executeQuery($sql, $params);
    }
    
    public function reopenPos($cdFilial, $cdCaixa, $dtAberCaix, $cdOperador) {
        $sql = "
            UPDATE TURCAIXA
                SET TO_DATE(:DTFECHCAIX, 'dd/mm/yyyy hh24:mi:ss') = :DTFECHCAIX,
                    CDOPERFECH = :CDOPERFECH
            WHERE CDFILIAL = :CDFILIAL
              AND CDCAIXA = :CDCAIXA
              AND DTABERCAIX = :DTABERCAIX
        ";
        
        $params = [
            ':CDFILIAL'   => $cdFilial,
            ':CDCAIXA'    => $cdCaixa,
            ':DTABERCAIX' => $dtAberCaix,
            ':CDOPERFECH' => null,
            ':DTFECHCAIX' => null
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function closePos($cdFilial, $cdCaixa, $dtAberCaix, $cdOperador) {
        $sql = "
            UPDATE TURCAIXA
                SET TO_DATE(:DTFECHCAIX, 'dd/mm/yyyy hh24:mi:ss') = :DTFECHCAIX,
                    CDOPERFECH = :CDOPERFECH
            WHERE CDFILIAL = :CDFILIAL
              AND CDCAIXA = :CDCAIXA
              AND DTABERCAIX = :DTABERCAIX
        ";
        
        $params = [
            ':CDFILIAL'   => $cdFilial,
            ':CDCAIXA'    => $cdCaixa,
            ':DTABERCAIX' => $dtAberCaix,
            ':CDOPERFECH' => $cdOperador,
            ':DTFECHCAIX' => date('d/m/Y H:i:s')
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function validateDuplicatedSale($cdFilial, $cupomNumber, $chaveSefaz) {
        $sql = "
            SELECT NRSEQVENDA FROM VENDA
             WHERE CDFILIAL = :CDFILIAL
              AND NRNOTAFISCALCE = :NRNOTAFISCALCE
              AND NRACESSONFCE = :NRACESSONFCE
        ";
        
        $params = [
            ':CDFILIAL'       => $cdFilial,
            ':NRNOTAFISCALCE' => $cupomNumber,
            ':NRACESSONFCE'   => $chaveSefaz
        ];
        
        return $this->connection->executeQuery($sql, $params);
    }
    
    public function insertSale($sale, $nrSeqVenda, $cdFilial, $cdLoja, $cdCaixa, $cdOperador, $nrOrg, $generalParams, $dtAberCaix, $dtEntrVenda, $xmlDecode, $noteDetails, $isNFCe, $isApprovedNFCe, $idStatusNFCe, $nrProtcoloNFCe, $dsObsStatusNFCe, $dateProtoc, $dtHrProtoNFCe, $idXmlValidado) {
        
        $sql = "
            INSERT INTO VENDA 
                ( CDFILIAL, CDLOJA, CDCAIXA, DTENTRVENDA, DTVENDA, DTABERVENDA, DTFECHAVENDA, CDEMPRESA, SGESTADO,
                  IDIMPVENDA, DTABERTUR, CDFILIALTUR, CDCAIXATUR, VRDESCVENDA, CDCLIENTE, CDCCUSCLIE, IDSITUVENDA,
                  NRSEQVENDA, NRORG, IDTPEMISVEND, CDOPERADOR, NRINSJURIEST, CDCONSUMIDOR, NMCONSVEND, NRINSCRCONS,
                  IDORIGEMVENDA, DTHRREALCONCONS, CDPREFINTGCOLVEN, IDTIPOVENDA, IDVENDEBCONS, NRPESMESAVENDA,
                  CDSENHAPED, CDVENDEDOR, NRVENDAREST, NRCOMANDAVND, NRMESA, VRTXSEVENDA, DSCOMANDAVND, VRTOTVENDA,
                  DTHRMESAFECHVEN, DSOBSDESC, CDGRPOCORDESC, DSOBSFINVEN, DSBAIRRO, IDRETBALLOJA, DSAREAATENDVEN,
                  NRCEPCONSVENDA, CDBAIRRO, CDMUNICIPIO, DSENDECONSVENDA, CDSERIENFCE, IDTPAMBNFCE, NRACESSONFCE,
                  NRNOTAFISCALCE, NRLANCTONFCE, IDSTATUSNFCE, DTEMISSAONFCE, NRPROTOCOLONFCE, DSOBSSTATUSNFCE,
                  IDMODOPERACNFCE, NRRECIBONFCE, CDOPERENVIONFCE, DTHRPROTOCONFCE, IDXMLVALIDNFCE, DSQRCODENFCE,
                  DSARQXMLNFCE, CDVERSXMLNFCVND
                ) 
                VALUES 
                ( :CDFILIAL, :CDLOJA, :CDCAIXA, TO_DATE(:DTENTRVENDA, 'dd/mm/yyyy'), TO_DATE(:DTVENDA, 'dd/mm/yyyy hh24:mi:ss'),
                  TO_DATE(:DTABERVENDA, 'dd/mm/yyyy hh24:mi:ss'), TO_DATE(:DTFECHAVENDA, 'dd/mm/yyyy hh24:mi:ss'), :CDEMPRESA,
                  :SGESTADO, :IDIMPVENDA, TO_DATE(:DTABERTUR, 'dd/mm/yyyy hh24:mi:ss'), :CDFILIALTUR, :CDCAIXATUR, :VRDESCVENDA,
                  :CDCLIENTE, :CDCCUSCLIE, :IDSITUVENDA, :NRSEQVENDA, :NRORG, :IDTPEMISVEND, :CDOPERADOR, :NRINSJURIEST, :CDCONSUMIDOR, 
                  :NMCONSVEND, :NRINSCRCONS, :IDORIGEMVENDA, TO_DATE(:DTHRREALCONCONS, 'dd/mm/yyyy hh24:mi:ss'), :CDPREFINTGCOLVEN,
                  :IDTIPOVENDA, :IDVENDEBCONS, :NRPESMESAVENDA, :CDSENHAPED, :CDVENDEDOR, :NRVENDAREST, :NRCOMANDAVND, :NRMESA,
                  :VRTXSEVENDA, :DSCOMANDAVND, :VRTOTVENDA, :DTHRMESAFECHVEN, :DSOBSDESC, :CDGRPOCORDESC, :DSOBSFINVEN, :DSBAIRRO,
                  :IDRETBALLOJA, :DSAREAATENDVEN, :NRCEPCONSVENDA, :CDBAIRRO, :CDMUNICIPIO, :DSENDECONSVENDA, :CDSERIENFCE,
                  :IDTPAMBNFCE, :NRACESSONFCE, :NRNOTAFISCALCE, :NRLANCTONFCE, :IDSTATUSNFCE, TO_DATE(:DTEMISSAONFCE, 'dd/mm/yyyy hh24:mi:ss'),
                  :NRPROTOCOLONFCE, :DSOBSSTATUSNFCE, :IDMODOPERACNFCE, :NRRECIBONFCE, :CDOPERENVIONFCE, TO_DATE(:DTHRPROTOCONFCE, 'dd/mm/yyyy hh24:mi:ss'),
                  :IDXMLVALIDNFCE, :DSQRCODENFCE, :DSARQXMLNFCE, :CDVERSXMLNFCVND
                )
        ";
        
        $params = [
			':CDFILIAL' 		=> $cdFilial,
			':CDLOJA' 		    => $cdLoja,
			':CDCAIXA' 		    => $cdCaixa,
			':DTENTRVENDA' 	    => date('d/m/Y', strtotime($dtEntrVenda)),
			':DTVENDA' 		    => date('d/m/Y H:i:s', strtotime($sale['saleDate'])),
			':DTABERVENDA' 	    => date('d/m/Y H:i:s', strtotime($sale['saleDate'])),
			':DTFECHAVENDA'     => date('d/m/Y H:i:s', strtotime($sale['saleDate'])),
			':CDEMPRESA' 	    => $generalParams['CDEMPRESA'],
			':SGESTADO'         => $generalParams['SGESTADO'],
			':IDIMPVENDA' 	    => 'S',
			':DTABERTUR' 	    => date('d/m/Y H:i:s', strtotime($dtAberCaix)),
			':CDFILIALTUR' 	    => $cdFilial,
			':CDCAIXATUR' 	    => $cdCaixa,
			':VRDESCVENDA' 	    => $this->util->tofloat($sale['totalDiscountAmount']),
			':CDCLIENTE' 	    => $generalParams['CDCLIENTE'], // Validar como será feita a identificação de consumidor da venda
			':CDCCUSCLIE' 	    => null, // Validar como será feita a identificação de consumidor da venda
			':IDSITUVENDA' 	    => strtolower($sale['saleStatus']) == 'concluida' ? 'O' : 'C',
			':NRSEQVENDA' 	    => $nrSeqVenda,
			':NRORG' 		    => $nrOrg,
			':IDTPEMISVEND'     => $generalParams['IDTPEMISSAOFOS'] == 'FNC'? 'N' : 'S',
			':CDOPERADOR' 	    => $cdOperador,
			':NRINSJURIEST'	    => $generalParams['NRINSJURIEST'],
			':CDCONSUMIDOR'	    => null,  // Validar como será feita a identificação de consumidor da venda
			':NMCONSVEND'	    => null,  // Validar como será feita a identificação de consumidor da venda
			':NRINSCRCONS'	    => null,  // Validar como será feita a identificação de consumidor da venda
			':IDORIGEMVENDA'    => 'DLV_INT',
			':DTHRREALCONCONS'  => date('d/m/Y H:i:s', strtotime($sale['saleDate'])),
			':CDPREFINTGCOLVEN' => 'TEK',
			':IDTIPOVENDA'	    => 'D',
			':IDVENDEBCONS'	    => 'N',
			':NRPESMESAVENDA'   => '1',
			':CDSENHAPED'       => null,
			':CDVENDEDOR'       => $generalParams['CDVENDEDOR'],
			':NRVENDAREST'      => null,
			':NRCOMANDAVND'     => null,
			':NRMESA'           => null,
			':VRTXSEVENDA'      => 0,
			':DSCOMANDAVND'     => null,
            ':VRTOTVENDA'       => str_replace('.', ',', $this->util->tofloat($sale['saleTotalAmount'])),
            ':DTHRMESAFECHVEN'  => null,
            ':DSOBSDESC'		=> null,
            ':CDGRPOCORDESC'    => null,
            ':DSOBSFINVEN'      => "sale-api-{$sale['noteSeries']}-{$sale['cupomNumber']}",
            ':DSBAIRRO'		    => null,
            ':IDRETBALLOJA'	    => null,
            ':DSAREAATENDVEN'   => null,
            ':NRCEPCONSVENDA'   => null,
            ':CDBAIRRO'	   	    => null,
            ':CDMUNICIPIO'	    => null,
            ':DSENDECONSVENDA'  => null,
            ':CDSERIENFCE'      => $sale['noteSeries'],
            ':IDTPAMBNFCE'      => $isNFCe ? $noteDetails['ide']['tpAmb'] : '1', 
            ':NRACESSONFCE'     => $sale['chaveSefaz'],
            ':NRNOTAFISCALCE'   => $generalParams['IDTPEMISSAOFOS'] == 'FNC'? str_pad($sale['cupomNumber'], 9, "0", STR_PAD_LEFT) : str_pad($sale['cupomNumber'], 6, "0", STR_PAD_LEFT),
            ':NRLANCTONFCE'     => $noteDetails['ide']['cNF'] ,
            ':IDSTATUSNFCE'     => isset($idStatusNFCe) ? $idStatusNFCe : null,
            ':DTEMISSAONFCE'    => date('d/m/Y H:i:s', strtotime($sale['emissionDate'])),
            ':NRPROTOCOLONFCE'  => $isNFCe ? $nrProtcoloNFCe : null,
            ':DSOBSSTATUSNFCE'  => $isNFCe ? $dsObsStatusNFCe: null,
            ':IDMODOPERACNFCE'  => $isNFCe ? '1' : null,
            ':NRRECIBONFCE'     => null,
            ':CDOPERENVIONFCE'  => null,
            ':DTHRPROTOCONFCE'  => $isNFCe ? date('d/m/Y H:i:s', strtotime($dtHrProtoNFCe)) : null,
            ':IDXMLVALIDNFCE'   => $isNFCe ? $idXmlValidado : null,
            ':DSQRCODENFCE'     => null,
            ':DSARQXMLNFCE'     => $xmlDecode,
            ':CDVERSXMLNFCVND'  => $isNFCe ? '4.00' : null
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function getSeqMovCaixa($cdFilial, $cdCaixa, $dtAberCaix) {
        $sql = "
            SELECT NVL(MAX(NRSEQUMOVI), 0) AS NRSEQUMOVI FROM MOVCAIXA
            WHERE CDFILIAL = :CDFILIAL
            AND CDCAIXA = :CDCAIXA
            AND DTABERCAIX = TO_DATE(:DTABERCAIX, 'DD/MM/YYYY HH24:MI:SS')
        ";
        $params = [
            ':CDFILIAL'   => $cdFilial,
            ':CDCAIXA'    => $cdCaixa,
            ':DTABERCAIX' => $dtAberCaix
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function insertMovCaixa($payment, $paymentDetails, $nrSeqVenda, $cdFilial, $cdLoja, $cdCaixa, $cdOperador, $nrOrg, $generalParams, $dtAberCaix, $dtEntrVenda, $nrSequMovi) {
        $sql = "INSERT INTO MOVCAIXA
            	( CDFILIAL, CDCAIXA, DTABERCAIX, NRSEQUMOVI,
                  DTHRINCMOV, IDTIPOMOVIVE, VRMOVIVEND,
                  NRSEQVENDA, NRORG, NRORGINCLUSAO, CDOPERINCLUSAO,
                  CDTIPORECE, DTMOVIMCAIXA, DSMOVIVEND, CDNSUHOSTTEF, NRAUTORIZATEF, CDCLIENTE
                )
            	VALUES
            	( :CDFILIAL, :CDCAIXA, TO_DATE(:DTABERCAIX, 'DD/MM/YYYY HH24:MI:SS'), :NRSEQUMOVI,
                  TO_DATE(:DTHRINCMOV, 'DD/MM/YYYY HH24:MI:SS'), :IDTIPOMOVIVE, :VRMOVIVEND,
                  :NRSEQVENDA, :NRORG, :NRORGINCLUSAO, :CDOPERINCLUSAO,
                  :CDTIPORECE, TO_DATE(:DTMOVIMCAIXA, 'DD/MM/YYYY HH24:MI:SS'), :DSMOVIVEND, :CDNSUHOSTTEF, :NRAUTORIZATEF, :CDCLIENTE
                )
        ";
            	
        $params = array(
            ':CDFILIAL' 	  => $cdFilial,
            ':CDCAIXA'  	  => $cdCaixa,
            ':DTMOVIMCAIXA'   => $dtAberCaix,
            ':NRSEQUMOVI'     => $nrSequMovi,
            ':IDTIPOMOVIVE'   => $payment["movimentType"],
            ':VRMOVIVEND'     => str_replace(".", ",", $this->util->tofloat($payment["Value"])),
            ':DTHRINCMOV'     => date('d/m/Y H:i:s'),
            ':NRSEQVENDA'     => $nrSeqVenda,
            ':CDTIPORECE'     => $paymentDetails["CDTIPORECE"],
            ':DTABERCAIX'     => $dtAberCaix,
            ':NRORG'          => $nrOrg,
            ':NRORGINCLUSAO'  => $nrOrg,
            ':CDOPERINCLUSAO' => $cdOperador,
            ':CDNSUHOSTTEF'   => !empty($payment['nsu']) ? $payment['nsu'] : null,
            ':NRAUTORIZATEF'  => !empty($payment['authorizationCode']) ? $payment['authorizationCode'] : null,
            ':CDCLIENTE'      => $generalParams['CDCLIENTE'],
            ':DSMOVIVEND'     => "Movimentação incluída através da integração de venda sale-api"
        );
    	
    	$this->connection->executeQuery($sql, $params);	
    }
    
    public function insertXML($cdFilial, $cdCaixa, $nrSeqVenda, $xml, $nrOrg) {
        $sql = "
            INSERT INTO ARQXMLNFCESAT 
                (CDFILIAL, CDCAIXA, NRSEQVENDA, DSARQXMLNFCE, NRORG, NRORGINCLUSAO, NRORGULTATU)
            VALUES 
                (:CDFILIAL, :CDCAIXA, :NRSEQVENDA, :DSARQXMLNFCE, :NRORG, :IDATIVO)
        ";
            	
        $params = array(
            ':CDFILIAL' 	=> $cdFilial,
            ':CDCAIXA'  	=> $cdCaixa,
            ':NRSEQVENDA'   => $nrSeqVenda,
            ':DSARQXMLNFCE' => $xml,
            ':NRORG'        => $nrOrg,
            ':IDATIVO'      => 'S'
        );
    	
    	$this->connection->executeQuery($sql, $params);	
    }
    
    public function updateMovClie($cdFilial, $cdCaixa, $nrSeqVenda, $dtAberCaix, $nrSequMovi, $cdTipoRece) {
        $sql = "
            UPDATE MOVICLIE
                SET 
                    CDCAIXA    = :CDCAIXA,
                    NRSEQVENDA = :NRSEQVENDA,
                    DTABERCAIX = :DTABERCAIX,
                    NRSEQUMOVI = :NRSEQUMOVI,
                    CDTIPORECE = :CDTIPORECE
            WHERE CDFILIAL = :CDFILIAL
             AND CDCLIENTE = :CDCLIENTE
             AND CDCONSUMIDOR = :CDCONSUMIDOR
             AND CDINTEMOVSALE = :CDINTEMOVSALE
             
                
                    
        ";
    }
    
    public function getConsumptionMovClie($cdFilial, $cdInteMovSale) {
        $sql = "
            SELECT MV.CDFILIAL, MV.CDCLIENTE, MV.CDCONSUMIDOR, MV.NRSEQMOVCLI,
                   MV.CDINTEMOVSALE, TO_CHAR(MV.DTMOVCLI, 'DD/MM/YYYY HH24:MI:SS'),
                   C.CDCCUSCLIE
            FROM MOVICLIE MV
                JOIN CONSUMIDOR C
                ON C.CDCLIENTE = MV.CDCLIENTE
                AND C.CDCONSUMIDOR = MV.CDCONSUMIDOR
            WHERE CDFILIAL = :CDFILIAL
             AND CDINTEMOVSALE = :CDINTEMOVSALE
        ";
        
        $params = [
            ':CDFILIAL'      => $cdFilial,
            ':CDINTEMOVSALE' => $cdInteMovSale
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function updateSale($cdFilial, $cdCaixa, $nrSeqVenda, $dtAberCaix, $cdCliente, $cdConsumidor, $cdCusClie) {
        $sql = "
            UPDATE VENDA
                SET 
                    CDCLIENTE    = :CDCLIENTE,
                    CDCONSUMIDOR = :CDCONSUMIDOR,
                    CDCCUSCLIE    = :CDCCUSCLIE
            WHERE CDFILIAL = :CDFILIAL
             AND CDCAIXA = :CDCAIXA
             AND NRSEQVENDA = :NRSEQVENDA
             AND DTABERTUR = :DTABERTUR
        ";
        
        $params = [
            ':CDFILIAL'     => $cdFilial,
            ':CDCAIXA'      => $cdCaixa,
            ':NRSEQVENDA'   => $nrSeqVenda,
            ':DTABERTUR'    => $dtAberCaix,
            ':CDCLIENTE'    => $cdCliente,
            ':CDCONSUMIDOR' => $cdConsumidor,
            ':CDCCUSCLIE'    => $cdCusClie
        ];
        
        $this->connection->executeQuery($sql, $params);
        
        $sql = "
            UPDATE MOVCAIXA
                SET 
                    CDCLIENTE = :CDCLIENTE
             WHERE CDFILIAL = :CDFILIAL
              AND CDCAIXA = :CDCAIXA
              AND NRSEQVENDA = :NRSEQVENDA
              AND DTABERCAIX = :DTABERCAIX
        ";
        
        $params = [
            ':CDFILIAL'     => $cdFilial,
            ':CDCAIXA'      => $cdCaixa,
            ':NRSEQVENDA'   => $nrSeqVenda,
            ':DTABERCAIX'   => $dtAberCaix,
            ':CDCLIENTE'    => $cdCliente
        ];
        
        $this->connection->executeQuery($sql, $params);
    }

}